/* Copyright (c) 2009 Arnaud Renevier, Inc, published under the modified BSD
 * license. */

OpenLayers.Control.SypAttribution = OpenLayers.Class (OpenLayers.Control.Attribution, {
    updateAttribution: function() {
        var attributions = [SypStrings.propulsedByLink];
        if (this.map && this.map.layers) {
            for(var i=0, len=this.map.layers.length; i<len; i++) {
                var layer = this.map.layers[i];
                if (layer.attribution && layer.getVisibility()) {
                    attributions.push( layer.attribution );
                }
            }  
            this.div.innerHTML = attributions.join(this.separator);
        }
    }
});

var SYP = {
    Settings: {
        MARKER_ICON: "openlayers/img/marker-blue.png",
        MARKER_ICON_HEIGHT: 25,
        MARKER_SELECT_ICON: "openlayers/img/marker-green.png",
        MARKER_SELECT_ICON_HEIGHT: 25
    },

    map: null,
    baseLayer: null,
    dataLayer: null,
    selectControl: null,

    init: function() {
        this.map = new OpenLayers.Map("map", {
            controls:[
                new OpenLayers.Control.SypAttribution(),
                new OpenLayers.Control.Navigation(),
                new OpenLayers.Control.PanZoom(),
                new OpenLayers.Control.Permalink()
            ],
            projection: new OpenLayers.Projection("EPSG:900913"),
            displayProjection: new OpenLayers.Projection("EPSG:4326")
        } );

        this.baseLayer = this.createBaseLayer();
        this.dataLayer = this.createDataLayer();
        this.map.addLayers([this.baseLayer, this.dataLayer]);

        this.selectControl = this.createSelectControl();
        this.map.addControl(this.selectControl);
        this.selectControl.activate();

        if (!this.map.getCenter()) {
            var centerBounds = new OpenLayers.Bounds();

            var mapProj = this.map.getProjectionObject();
            var sypOrigProj = new OpenLayers.Projection("EPSG:4326");

            var bottomLeft = new OpenLayers.LonLat(sypOrig[0],sypOrig[1]);
            bottomLeft = bottomLeft.transform(sypOrigProj, mapProj);
            var topRight = new OpenLayers.LonLat(sypOrig[2],sypOrig[3])
            topRight = topRight.transform(sypOrigProj, mapProj);

            centerBounds.extend(bottomLeft);
            centerBounds.extend(topRight);
            this.map.zoomToExtent(centerBounds);
        }
    },

    createBaseLayer: function() {
        return new OpenLayers.Layer.OSM("OSM");
    },

    createDataLayer: function(map) {
        var defaultStyle = new OpenLayers.Style({
            externalGraphic: this.Settings.MARKER_ICON,
            graphicHeight: "${height}"
        }, {
            context: {
                height: function(feature) {
                    var defaultHeight = SYP.Settings.MARKER_ICON_HEIGHT || 32;
                    var increase = 4 * (feature.attributes.count - 1);
                    return Math.min(defaultHeight + increase, 50);
                }
            }
        });
        var selectStyle = new OpenLayers.Style({
            externalGraphic: this.Settings.MARKER_SELECT_ICON,
            graphicHeight: this.Settings.MARKER_SELECT_ICON_HEIGHT || 32 
        });
        var styleMap = new OpenLayers.StyleMap (
                        {"default": defaultStyle,
                         "select": selectStyle});

        var layer = new OpenLayers.Layer.GML("KML", "items.php", 
           {
               strategies: [
                new OpenLayers.Strategy.Cluster()
                ],
            styleMap: styleMap,
            format: OpenLayers.Format.KML, 
            projection: this.map.displayProjection,
            eventListeners: { scope: this,
                              loadend: this.checkForFeatures
                            }
           });

        return layer;
    },

    createSelectControl: function() {
        var control = new OpenLayers.Control.SelectFeature(
                                            this.dataLayer, {
                                               onSelect: this.onFeatureSelect,
                                               onUnselect: this.onFeatureUnselect,
                                               toggle: true,
                                               clickout: false
                                                            });
        return control;
    },

    checkForFeatures: function() {
        var features = this.dataLayer.features;
        if (features.length == 0) {
            var message = SypStrings.noImageRegistered;
            this.Utils.displayUserMessage(message, "warn");
        }
    },

    createPopup: function(position, contentHTML) {
        var popup = new OpenLayers.Popup.Anchored("popup",
                                                  position,
                                                  null,
                                                  contentHTML,
                                                  null,
                                                  true);
        popup.autoSize = true;
        popup.backgroundColor = ""; // deal with it in css
        popup.border = ""; // deal with it in css
        popup.closeOnMove = true;
        return popup;
    },

    onFeatureUnselect: function (feature) {
        var map = feature.layer.map;
        var permaControl = map.getControlsByClass("OpenLayers.Control.Permalink");
        if (permaControl[0]) {
            permaControl[0].div.style.display = "";
        }
        if (!feature.popup) {
            this.map.events.unregister("movestart", this, this._unselect);
            return;
        }
        var popup = feature.popup;
        if (popup.visible()) {
            popup.hide();
        }
    },

    onFeatureSelect: function(feature) {
        var map = feature.layer.map;
        if (feature.attributes.count > 1) {
            this.unselect(feature);
            var lonlat = new OpenLayers.LonLat(feature.geometry.x, feature.geometry.y);
            map.setCenter(lonlat, map.zoom + 1);
            return;
        }
        var permaControl = map.getControlsByClass("OpenLayers.Control.Permalink");
        if (permaControl[0]) {
            permaControl[0].div.style.display = "none";
        }
        var popup = feature.popup;

        var brCorner = SYP.Utils.brCorner(map, 8);

        // we cannot reuse popup; we need to recreate it in order for IE
        // expressions to work. Otherwise, we get a 0x0 image on second view.
        if (popup) {
            popup.destroy();
        }
        var contentHTML;
        if (feature.cluster[0].attributes.name) {
            contentHTML = "<h2>" +
                          feature.cluster[0].attributes.name + 
                          "</h2>" + 
                          feature.cluster[0].attributes.description;
        } else {
            contentHTML = feature.cluster[0].attributes.description;
        }
        if (!contentHTML || !contentHTML.length) {
            this.map.events.register("movestart", this, this._unselect = function () { this.unselect(feature)});
            return;
        }
        popup = SYP.createPopup(brCorner, contentHTML);
        var control = this;
        popup.hide = function () {
            OpenLayers.Element.hide(this.div);
            control.unselectAll();
        };
        map.addPopup(popup);
        feature.popup = popup;
        var anchor = popup.div.getElementsByTagName("a")[0];
        if (anchor) {
            anchor.onclick = function() { 
                SYP.showBigImage(this.href);
                return false;
            }
        }
    },

    showBigImage: function (href) {
        try {
            document.getElementById('bigimg_container').style.display = "table";
        } catch(e) {
            document.getElementById('bigimg_container').style.display = "block";
        }
        var maxHeight = document.body.clientHeight * 0.9;
        var maxWidth = document.body.clientWidth * 0.9;
        document.getElementById('bigimg').style.height = "";
        document.getElementById('bigimg').style.width = "";
        document.getElementById('bigimg').style.maxHeight = maxHeight + "px";
        document.getElementById('bigimg').style.maxWidth = maxWidth + "px";
        document.getElementById('bigimg').onload = function () {
            var icon = document.getElementById('bigimg_close');
            icon.style.top = this.offsetTop;
            icon.style.left = this.offsetLeft + this.clientWidth - icon.clientWidth;

            var heightRatio = this.clientHeight / parseInt(this.style.maxHeight);
            var widthRatio = this.clientWidth / parseInt(this.style.maxWidth);
            if (heightRatio > 1 || widthRatio > 1) {
                if (heightRatio > widthRatio) {
                    this.style.height = this.style.maxHeight;
                } else {
                    this.style.width = this.style.maxWidth;
                }
            }

        };
        document.getElementById('bigimg').src = href;
    },

    closeBigImage: function() {
        document.getElementById('bigimg').src = "";
        document.getElementById('bigimg').parentNode.innerHTML = document.getElementById('bigimg').parentNode.innerHTML;
        document.getElementById('bigimg_container').style.display = "none";
    },

    Utils: {
        brCorner: function(map, margin) {
            var bounds = map.calculateBounds();
            var corner = new OpenLayers.LonLat(bounds.right, bounds.bottom);
            var cornerAsPx = map.getPixelFromLonLat(corner);
            cornerAsPx = cornerAsPx.add( -margin, -margin);
            corner = map.getLonLatFromPixel(cornerAsPx);
            return corner;
        },
        displayUserMessage: function(message, status) {
            var div = document.getElementById('message');
            while (div.firstChild)
                div.removeChild(div.firstChild);
            var textNode = document.createTextNode(message);
            switch (status) {
                case "error":
                    div.style.color = "red";
                    break;
                case "warn":
                    div.style.color = "#FF8C00";
                    break;
                case "success":
                    div.style.color = "green";
                    break;
                default:
                    div.style.color = "black";
                    break;
            }
            div.style.display = "block";
            div.appendChild(textNode);
        }
    }
};

// if possible, determine language with HTTP_ACCEPT_LANGUAGE instead of
// navigator.language
if (OpenLayers.Lang[SypStrings.language]) {
    OpenLayers.Lang.setCode(SypStrings.language);
}

// avoid alerts
OpenLayers.Console.userError = function(error) { 
    SYP.Utils.displayUserMessage(error, "error");
}

// sometimes, especially when cache is clear, firefox does not compute
// correctly popup size. That's because at the end of getRenderedDimensions,
// dimensions of image is not known. So, we work around that problem by setting
// image width and image height. That way, dimensions of image are set in
// innerHTML, and are therefore known in getRenderedDimensions
OpenLayers.Popup.Anchored.prototype.registerImageListeners = function() {
    var onImgLoad = function() {
        this.img.width =  this.img.width;
        this.img.height =  this.img.height;
        this.popup.updateSize();
        OpenLayers.Event.stopObserving(
            this.img, "load", this.img._onImageLoad
        );
    };

    var images = this.contentDiv.getElementsByTagName("img");
    for (var i = 0, len = images.length; i < len; i++) {
        var img = images[i];
        if (img.width == 0 || img.height == 0) {

            var context = {
                'popup': this,
                'img': img
            };

            img._onImgLoad = OpenLayers.Function.bind(onImgLoad, context);

            OpenLayers.Event.observe(img, 'load', img._onImgLoad);
        }    
    } 
}
